package classes
{
	import com.stepnetworks.remotes.exercise.ExerciseWordScore;
	import com.stepnetworks.remotes.user.CustomizationData;
	import com.stepnetworks.remotes.user.User;
	
	import flash.events.AsyncErrorEvent;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.NetStatusEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.NetConnection;
	import flash.net.NetStream;
	import flash.net.Responder;
	import flash.net.SharedObject;
	
	import mx.controls.Alert;
	
	[Bindable]
	public class DataAccess
	{
		[Embed(source="../resources/audio/smallClapping.mp3")]
		public static var SOUND_SMALL_CLAP : Class;
		[Embed(source="../resources/audio/bigClapping.mp3")]
		public static var SOUND_BIG_CLAP : Class;
		
		private var fNetConn : NetConnection;
		private var fLoggedInUser: User;
		private var fConnectionStream : NetStream; 
		
		public function DataAccess(RTMPHost:String)
		{
			NetConnection.defaultObjectEncoding = flash.net.ObjectEncoding.AMF3;
			SharedObject.defaultObjectEncoding = flash.net.ObjectEncoding.AMF3;			
				
			fNetConn = new NetConnection();
			fNetConn.addEventListener( NetStatusEvent.NET_STATUS, _NetStatusHandler );
			fNetConn.addEventListener( SecurityErrorEvent.SECURITY_ERROR, _NetErrorHandler );
			fNetConn.addEventListener( AsyncErrorEvent.ASYNC_ERROR, _NetErrorHandler );
			fNetConn.addEventListener( IOErrorEvent.IO_ERROR, _NetErrorHandler );
			fNetConn.connect("rtmp://" + RTMPHost + "/ssi-rtf");
		}
		
		private function _NetStatusHandler(event:NetStatusEvent) : void
		{
			trace ("NetStatus: " + event.info["code"]);
			
			switch (event.info["code"])
			{
				case "NetConnection.Connect.Success":
					trace("Connection opened");
				break;
				case "NetConnection.Connect.Rejected":
					trace("Connection rejected");
				break;
				case "NetConnection.Connect.Closed":
					trace("Connection close");
					Alert.show("An error occured while communicating with the server.  Refreshing your browser may resolve the trouble.");
				break;
			}
		}
		
		private function _NetErrorHandler(event : Event) : void
		{
			classes.Utils.showError("Network Error: " + event.toString());
		}
		
		private function _loginResult(result : User) : void
		{
			fLoggedInUser = result;
		}
		
		//return the RTMP stream configured for this connection
		public function getConnectionStream() : NetStream
		{
			if (fConnectionStream == null)
				fConnectionStream = new NetStream(fNetConn);
				
			return fConnectionStream;
		}
		
		/***
		 *  User management functions 
		 */
		public function getAllUsers(cbHandler : Responder) : void
		{			
			fNetConn.call("getAllUsers", cbHandler);
		}
		
		public function addUser(user: User, cbHandler: Responder):void
		{
			fNetConn.call("addUser", cbHandler, user);
		}
		
		public function deleteUser(user : User, cbHandler : Responder) : void
		{
			fNetConn.call("deleteUser", cbHandler, user);
		}
		
		public function updateUser(user : User, cbHandler : Responder) : void
		{
			fNetConn.call("updateUser", cbHandler, user);
		}
		
		public function canDeleteUser(user : User, cbHandler : Responder) : void
		{
			fNetConn.call("canDeleteUser", cbHandler, user);
		}
		
		public function updateUserPassword(user : User, cbHandler : Responder) : void
		{
			fNetConn.call("updateUserPassword", cbHandler, user);
		}
		
		public function doLogin(Username : String, Password : String, cbHandler : Responder) : void
		{		
			fNetConn.call("doLogin", cbHandler, Username, Password);
		}
		
		public function loadCustomizationOptions(user : User, cbHandler : Responder) : void
		{
			fNetConn.call("loadCustomizationOptions", cbHandler, user);
		}
		
		public function saveCustomizationOptions(user : User, options : CustomizationData, cbHandler : Responder) : void
		{
			fNetConn.call("saveCustomizationOptions", cbHandler, user, options);
		}
		
		public function beginStreamRecording(StreamName : String, BroadcastName : String) : void
		{
			fNetConn.call("clientStreamFunctions.beginStreamRecording", null, StreamName, BroadcastName);
		}
		
		public function stopStreamRecording(StreamName : String, BroadcastName : String) : void
		{
			fNetConn.call("clientStreamFunctions.stopStreamRecording", null, StreamName, BroadcastName);
		}
		
		//retrieve an exercise list
		public function getAllExercises(user : User, cbHandler : Responder) : void
		{
			fNetConn.call("getAllExercises", cbHandler);
		}
		
		//load exercise parameters
		public function loadExerciseParameters(ExerciseID : String, cbHandler : Responder) : void
		{
			fNetConn.call("loadExerciseParameters", cbHandler, ExerciseID);
		}
		
		public function loadExerciseCheckpoints(ExerciseID : String, cbHandler : Responder) : void
		{
			fNetConn.call("loadExerciseCheckpoints", cbHandler, ExerciseID);
		}
		
		public function loadExerciseWords(ExerciseID : int, cbHandler : Responder) : void
		{
			fNetConn.call("getExerciseWords", cbHandler, ExerciseID);
		}
		
		public function submitWordScore(score : ExerciseWordScore) : void
		{
			if (score.ExerciseWordID > 0)
				fNetConn.call("submitWordScore", null, score);
		}
		
		public function getUserWordScores(usr:  User, cbHandler : Responder) : void
		{
			fNetConn.call("getUserWordScores", cbHandler, usr);
		}
	}
}