package components
{
	import flash.display.Graphics;
	import flash.events.Event;
	
	import mx.containers.Canvas;
	import mx.events.FlexEvent;
	
	public class CartesianCanvas extends Canvas
	{
		private var f0PosX : Number;
		private var f0PosY : Number;
		private var fMaxX : Number;
		private var fMinX : Number;
		private var fMaxY : Number;
		private var fMinY : Number;
		
		private var fPenSize : int = 1;
		private var fPenColor : uint = 0xcccccc;
		
		private var fShowLabels : Boolean = false;
		private var fShowOriginLines : Boolean = true;
		
		private function onCreate() : void
		{
			redraw();
		}
		
		private function onResize() : void
		{
			_remeasure();
		}
		
		private function onCreationComplete(event : Event) : void
		{
			redraw();
		}
		
		private function redraw() : void
		{
			//_drawCanvasBackground();
			
			if (fShowOriginLines)
				_drawOriginGridLines();
				
			//if (fShowAxisLabels)
			//
			
			//if (fShowAxisExtentsLabels)
			//
		}
		
		private function _remeasure() : void
		{
			f0PosX = this.width / 2;
			f0PosY = this.height / 2;
			
			fMaxX = this.width / 2;
			fMinX = nabs(this.width / 2);
			
			fMaxY = this.height / 2;
			fMinY = nabs(this.height /2 );
		}
		
		private function _drawCanvasBackground() : void
		{
			var g : Graphics = this.graphics;
			var bgColor : uint = this.getStyle("backgroundColor") as uint;
			
			g.lineStyle(1, bgColor);
			
			g.beginFill(bgColor);
			g.drawRect(0, 0, this.width, this.height);
			g.endFill();
			
		}
		
		private function _drawOriginGridLines() : void
		{
			var g : Graphics = this.graphics;
			
			g.lineStyle(2, 0xffffff);
			 
			//plot a line at the center points
			g.moveTo(this.width / 2, 1);
			g.lineTo(this.width / 2, this.height - 1);
			
			g.moveTo(1, this.height / 2);
			g.lineTo(this.width - 1, this.height / 2);	
		}
		
		public function CartesianCanvas()
		{
			this.setStyle("backgroundColor", 0x222222);
			
			this.addEventListener(FlexEvent.CREATION_COMPLETE, onCreationComplete);
		}
		
		public function putPen(x:int, y : int) : void
		{
			this.graphics.moveTo(x, y);
		}
		
		public function lineTo(x : int, y : int) : void
		{
			this.graphics.lineStyle(fPenSize, fPenColor);
			this.graphics.lineTo(x, y);
		}
		
		public function putLine(x1 : Number, y1 : Number, x2 : Number, y2 : Number) : void
		{
			var g : Graphics = this.graphics;
			
			g.lineStyle(fPenSize, fPenColor);
			
			g.moveTo(x1, y1);			
			
			g.lineTo(x2, y2);
		}
		
		public function putPoint(x : Number, y : Number) : void
		{
			var g : Graphics = this.graphics;
			
			g.lineStyle(fPenSize, fPenColor);
			
			if ((x > this.width) || (x < 0) || ((y > this.height) || (y < 0)))
				return;
			
			g.moveTo(x, y);
			
			g.lineTo(x + 1, y + 1);
		}
					
		public function plotPoint(CartX : Number, CartY : Number) : void
		{
			var drawX : Number;
			var drawY : Number;
			
			drawX = f0PosX + CartX;
			drawY = f0PosY + CartY;
			
			putPoint(drawX, drawY);
		}
		
		public function resetToOrigin() : void
		{
			this.graphics.moveTo(0, 0);
		}
		
		public function resetToCartOrigin() : void
		{
			this.graphics.moveTo(f0PosX, f0PosY);
		}
		
		public function clear() : void
		{
			this.graphics.clear();
			redraw();
		}
		
		public function nabs(val : Number) : Number
		{
			return val - (val * 2); 
		}
		
		public function getMaxX() : int
		{
			return this.width / 2;
		}
		
		public function getMinX() : int
		{
			return fMinX;
		}
		
		public function getMaxY() : int
		{
			return this.height / 2;
		}
		
		public function getMinY() : int
		{
			return fMinY;
		}
		
		public function set penSize(val : int) : void
		{
			fPenSize = val;
		}
		
		public function get penSize() : int
		{
			return fPenSize;
		}
		
		public function set penColor(color : uint) : void
		{
			fPenColor = color;
		}
		
		public function get penColor() : uint
		{
			return fPenColor;
		}
		
				public function set showOriginLines(State : Boolean) : void
		{
			fShowOriginLines = State;
		}
		
		public function get showOriginLines() : Boolean
		{
			return fShowOriginLines;
		}
	}
}