package components
{
	import flash.events.Event;
	
	import mx.containers.Canvas;
	import mx.controls.TextArea;
	import mx.events.FlexEvent;
	import mx.events.ResizeEvent;
	
	public class SpeechBalloon extends Canvas {
		private static var BORDER_PADDING : int = 15;
		private static var TAIL_OFFSET : int = 5;
		private static var SHADOW_OFFSET : int = 5;
		private static var CORNER_SIZE : int = 32;
		
		private var fTailSide : int;
		private var fTailY : int;
		private var fTailX : int;
		
		private var fPaddingTop : int;
		private var fPaddingBottom : int;
		private var fPaddingLeft : int;
		private var fPaddingRight : int;
		
		private var fInternalText : TextArea;
		
		public function SpeechBalloon() {
			super();
			fInternalText = new TextArea();
			fInternalText.editable = false;
			
			addEventListener(FlexEvent.CREATION_COMPLETE, onCreationComplete);
		}
		
		private function onCreationComplete(event : Event) : void {
			fInternalText.visible = true;
			fInternalText.setStyle("borderThickness", "0");
			__resizeInternal();
			this.addChild(fInternalText);
		}

		override protected function measure() : void {
			super.measure();
			minWidth = 20;
			measuredWidth = 320;
			minHeight = 20;
			measuredHeight = 200;
		}

		
		private function onResize(event : ResizeEvent) : void {
			__resizeInternal();
		}
		
		private function __resizeInternal() : void {
			fInternalText.width = width - (BORDER_PADDING * 2) - fPaddingRight;
			fInternalText.height = height - (BORDER_PADDING * 2) - fPaddingBottom;
			fInternalText.x = BORDER_PADDING + fPaddingLeft;
			fInternalText.y = BORDER_PADDING + fPaddingTop;
		}		
		
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void {
			var leftShadowOffset : int = 0;
			var rightShadowOffset : int = 0;
			var topShadowOffset : int = 0;
			var bottomShadowOffset : int =0;
			
			var leftBalloonOffset : int = 0;
			var topBalloonOffset : int = 0;
			var rightBalloonOffset : int = 0;
			var bottomBalloonOffset : int = 0;
			
			switch (fTailSide) {
				//right side shadow
				case 0:
					leftShadowOffset = TAIL_OFFSET + SHADOW_OFFSET;
					rightShadowOffset = 0;
					topShadowOffset = SHADOW_OFFSET;
					
					bottomShadowOffset = 0;
					
					leftBalloonOffset = TAIL_OFFSET;
					rightBalloonOffset = SHADOW_OFFSET;
					topBalloonOffset = 0;
					bottomBalloonOffset = SHADOW_OFFSET;
				break;
				
				//top shadow
				
				//left side shadow
				
				//bottom shadow
			}
			
			with (graphics) {
		
				//draw background/shadow
				beginFill(0x000000, 1);
				drawRoundRect(leftShadowOffset, topShadowOffset, (unscaledWidth - leftShadowOffset - rightShadowOffset), (unscaledHeight - topShadowOffset - bottomShadowOffset), CORNER_SIZE);
				endFill();
				
				//draw outline around entire component
				lineStyle(2, 0x000000, 1);
				//draw foreground/balloon
				beginFill(0xffffff, 1);
				drawRoundRect(leftBalloonOffset, topBalloonOffset, (unscaledWidth - leftBalloonOffset - rightBalloonOffset), (unscaledHeight - topBalloonOffset - bottomBalloonOffset), CORNER_SIZE);
				endFill();
			}
			
			super.updateDisplayList(unscaledWidth, unscaledHeight);
		}
		
		public function get text() : String {
			return fInternalText.htmlText;
		}
		
		public function set text(NewText : String) : void {
			fInternalText.htmlText = NewText;
		}
		
		public function set tailX(NewX : int) : void {
			if ((NewX >= 0) && (NewX <= this.width))
				fTailX = NewX;
		}
		
		public function set tailY(NewY : int) : void {
			if ((NewY >= 0) && (NewY <= this.height))
				fTailY = NewY;
		}
		
		public function set tailSide(NewSide : int) : void {
			fTailSide = NewSide;
		}
		
		public function set PaddingTop(Val : int) : void {
			fPaddingTop = Val;
		}
		
		public function get PaddingTop() : int {
			return fPaddingTop;
		}
		
		public function set PaddingLeft(Val : int) : void {
			fPaddingLeft = Val;
		}
		
		public function get PaddingLeft() : int {
			return fPaddingLeft;
		}
		
		public function set PaddingBottom(Val : int) : void {
			fPaddingBottom = Val;
		}
		
		public function get PaddingBottom() : int {
			return fPaddingBottom;
		}
		
		public function set PaddingRight(Val : int) : void {
			fPaddingRight = Val;
		}
		
		public function get PaddingRight() : int {
			return fPaddingRight;
		}
	}
}