package com.stepnetworks.data;

import java.sql.Connection;
import java.util.logging.Level;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import com.stepnetworks.Utility;

public class DataAccess {
	private Connection fDBConn;
	private String fJDBC_URL;
	private HashMap<String, PreparedStatement> fStatements;

	/***
	 * DataAccess constructor
	 * @param DBName
	 * @param DBHost
	 * @param DBPort
	 * @param DBUser
	 * @param DBPassword
	 */
	public DataAccess(String DBName, String DBHost, int DBPort, String DBUser,
			String DBPassword) {
		try {
			//load the driver
			Class.forName("com.mysql.jdbc.Driver").newInstance();
			fJDBC_URL = "jdbc:mysql://" + DBHost + "/" + DBName + "?user=" + DBUser + "&password=" + DBPassword;
			fStatements = new HashMap<String, PreparedStatement>();

			//reconnect handles rebuilding the prepared statements
			_reconnectDB();
		} catch (Exception ex) {
			Utility.log(Level.SEVERE, "An error occured while setting up the database connection, and registering the runtime cleanup handler: " + ex.getMessage());
		}
	}
	
	public void _reconnectDB()
	{
		try
		{
			if ((fDBConn == null) || fDBConn.isClosed())
			{
				fDBConn = DriverManager.getConnection(fJDBC_URL);
				_prepareStatements();
			}
		} catch (SQLException sqlex)
		{
			Utility.log(Level.SEVERE, "An error occured while initiating a database connection: " + sqlex.getMessage());
		}
	}
	
	public void close()
	{
		try
		{
			fDBConn.close();
		} catch (SQLException sqlex)
		{
			Utility.log(Level.SEVERE, "An error occured while closing the database connection: " + sqlex.getMessage());
		}
	}

	private void _prepareStatements() {
		String StmntName = "";
		PreparedStatement Stmnt = null;

		try {
			if (fStatements.size() > 0)
			{
				//throw away the compiled statements, and rebuild them
				for (int i = 0; i < fStatements.size(); i++)
				{
					Collection<PreparedStatement> items = fStatements.values();
					
					for (PreparedStatement stmntItem : items)
					{
						stmntItem.close();
					}
				}
				
				fStatements.clear();
			}
			
			//user login test
			Stmnt = fDBConn.prepareStatement("SELECT UserID, Username, Password, FirstName, LastName, Enabled FROM users WHERE Username=? AND Password=?");
			fStatements.put("getUserByLoginDetails", Stmnt);

			//get user list
			Stmnt = fDBConn.prepareStatement("SELECT UserID, Username, FirstName, LastName, Enabled FROM users");
			fStatements.put("getAllUsers", Stmnt);

			//get user by user id
			Stmnt = fDBConn.prepareStatement("SELECT UserID, Username, Password, FirstName, LastName, Enabled FROM users WHERE UserID=?");
			fStatements.put("getUserByID", Stmnt);

			//add user
			Stmnt = fDBConn.prepareStatement("INSERT INTO users (Username, Enabled, FirstName, LastName) VALUES(?, ?, ?, ?)", PreparedStatement.RETURN_GENERATED_KEYS);
			fStatements.put("addUser", Stmnt);

			//delete user
			Stmnt = fDBConn.prepareStatement("DELETE FROM users WHERE UserID=?");
			fStatements.put("deleteUser", Stmnt);

			//update user
			Stmnt = fDBConn.prepareStatement("UPDATE users SET Username=?, Enabled=?, FirstName=?, LastName=? WHERE UserID=?");
			fStatements.put("updateUser", Stmnt);
			
			//update user password
			Stmnt = fDBConn.prepareStatement("UPDATE users SET Password=? WHERE Userid=?");
			fStatements.put("updateUserPassword", Stmnt);
			
			//update user customization selections
			Stmnt = fDBConn.prepareStatement("UPDATE users SET Nickname=?, Gender=?, Hairstyle=?, SkinColor=?, OutfitColor=? WHERE UserID=?");
			fStatements.put("saveCustomizationOptions", Stmnt);
			
			Stmnt = fDBConn.prepareStatement("INSERT INTO user_wordscores (UserID, Score, WordID) VALUES(?, ?, ?)");
			fStatements.put("submitWordScore", Stmnt);
			
			Stmnt = fDBConn.prepareStatement("SELECT Nickname, Gender, Hairstyle, SkinColor, OutfitColor FROM users WHERE UserID=?");			
			fStatements.put("loadCustomizationOptions", Stmnt);
			
			//retrieve all exercise info
			Stmnt = fDBConn.prepareStatement("SELECT ExerciseID, Name, Description, SyllabicDuration, ExerciseType FROM exercises");
			fStatements.put("getAllExercises", Stmnt);
			
			Stmnt = fDBConn.prepareStatement("SELECT WordID, Text, SyllableCount FROM exercise_words WHERE ExerciseID=?");
			fStatements.put("getExerciseWords", Stmnt);
			
			Stmnt = fDBConn.prepareStatement("SELECT count(WordID) FROM user_wordscores WHERE UserID=?");
			fStatements.put("canDeleteUser", Stmnt);
			
			Stmnt = fDBConn.prepareStatement("SELECT WordScoreID, Score, user_wordscores.WordID, UserID, `Text`, Name FROM user_wordscores LEFT JOIN exercise_words ON user_wordscores.WordID=exercise_words.WordID LEFT JOIN exercises ON exercise_words.ExerciseID=exercises.ExerciseID WHERE UserID=?");
			fStatements.put("getUserWordScores", Stmnt);
			
		} catch (Exception ex) {
			Utility.log(Level.SEVERE, "Errors occured while preparing statement [" + StmntName + "] : " + ex.getMessage());
		}
	}

	/***
	 * Retrieve a prepared statement for a given function.  Null if there is no statement available.
	 * @param SQLStatement
	 * @return
	 */
	public PreparedStatement getStatementFor(String SQLStatement)
	{
		_reconnectDB();
		PreparedStatement result = fStatements.get(SQLStatement);
		return result; 
	}
}
