package com.stepnetworks.data;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.logging.Level;

import com.stepnetworks.Utility;
import com.stepnetworks.flex.remotes.CustomizationOptions;
import com.stepnetworks.flex.remotes.ExerciseWordScore;
import com.stepnetworks.flex.remotes.User;

public class UserFunctions {
	private DataAccess fDBConn;

	public UserFunctions(DataAccess DBConn) {
		fDBConn = DBConn;
	}

	/***
	 * Retrieve a list of all the systems users
	 * @return
	 */
	public ArrayList<User> getAllUsers() {
		ArrayList<User> results = new ArrayList<User>();

		PreparedStatement stmnt = fDBConn.getStatementFor("getAllUsers");

		try {
			if (stmnt.execute()) {
				ResultSet rs = stmnt.getResultSet();
				while (rs.next()) {
					//for each row, try creating a user
					User tmpUser = new User();
					tmpUser.setUserID(rs.getInt("UserID"));
					tmpUser.setUsername(rs.getString("Username"));
					tmpUser.setFirstName(rs.getString("FirstName"));
					tmpUser.setLastName(rs.getString("LastName"));
					tmpUser.setEnabled(rs.getBoolean("Enabled"));

					results.add(tmpUser);
				}

				rs.close();
			}
		} catch (SQLException sqlex) {
			Utility.log(Level.SEVERE, "Failed retrieving user listings : " + sqlex.getMessage());
		}

		return results;
	}

	/***
	 * Retrieve a user by the login details
	 * @param Username
	 * @param Password
	 * @return
	 */
	public User getUserByLoginDetails(String Username, String Password) {
		User result = new User();

		PreparedStatement stmnt = fDBConn
				.getStatementFor("getUserByLoginDetails");

		try {
			stmnt.setString(1, Username);
			stmnt.setString(2, Password);

			if (stmnt.execute()) {
				ResultSet rs = stmnt.getResultSet();
				if (rs.next()) {
					//for each row, try creating a user
					result.setUserID(rs.getInt("UserID"));
					result.setUsername(rs.getString("Username"));
					result.setFirstName(rs.getString("FirstName"));
					result.setLastName(rs.getString("LastName"));
					result.setPassword(rs.getString("Password"));
					result.setEnabled(rs.getBoolean("Enabled"));
				} else {
					result.setUserID(-1);
					result.setUsername("invalid");
					result.setPassword("");
					result.setEnabled(false);
				}

				rs.close();
			}
		} catch (SQLException sqlex) {
			Utility.log(Level.SEVERE, "Failed to lookup user by login details : " + sqlex.getMessage());
		}

		return result;
	}

	/***
	 * Update a users details
	 * @param user
	 * @return
	 */
	public boolean updateUser(User user) {
		boolean result = false;

		PreparedStatement stmnt = fDBConn.getStatementFor("updateUser");

		try {
			stmnt.setString(1, user.getUsername());
			stmnt.setBoolean(2, user.getEnabled());
			stmnt.setString(3, user.getFirstName());
			stmnt.setString(4, user.getLastName());
			stmnt.setInt(5, user.getUserID());

			stmnt.execute();
			result = true;
		} catch (SQLException sqlex) {
			Utility.log(Level.SEVERE, "Failed to update user details : " + sqlex.getMessage());
		}

		return result;
	}

	/***
	 * Remove a user from the system.  The user cannot be removed, if there is exercise data for them.
	 * @param user
	 * @return
	 */
	public boolean deleteUser(User user) {
		boolean result = false;

		if (canDeleteUser(user)) {
			PreparedStatement stmnt = fDBConn.getStatementFor("deleteUser");
			try {
				//can delete user?
				stmnt.setInt(1, user.getUserID());
				stmnt.execute();
				result = true;
			} catch (SQLException sqlex) {
				Utility.log(Level.SEVERE, "Failed to remove user : " + sqlex.getMessage());
			}
		}

		return result;
	}

	/***
	 * Add a new user to the system
	 * @param user
	 * @return
	 */
	public User addUser(User user) {
		User result = null;
		
		if (user.getUserID() > 0)
			return user;
		
		PreparedStatement stmnt = fDBConn.getStatementFor("addUser");

		try {
			stmnt.setString(1, user.getUsername());
			stmnt.setBoolean(2, user.getEnabled());
			stmnt.setString(3, user.getFirstName());
			stmnt.setString(4, user.getLastName());

			stmnt.execute();
			
			ResultSet rs = stmnt.getGeneratedKeys();
			result = new User();

			if (rs.next())
				result.setUserID(rs.getInt(1));
				
			rs.close();
		} catch (SQLException sqlex) {
			Utility.log(Level.SEVERE, "Failed to add new user : " + sqlex.getMessage());
		}

		return result;
	}

	/***
	 * Update or set a users login password
	 * @param user
	 * @return
	 * @throws SQLException
	 */
	public boolean updateUserPassword(User user) {
		boolean result = false;

		PreparedStatement stmnt = fDBConn.getStatementFor("updateUserPassword");

		try {
			stmnt.setString(1, user.getPassword());
			stmnt.setInt(2, user.getUserID());
			stmnt.execute();
			result = true;
		} catch (SQLException sqlex) {
			Utility.log(Level.SEVERE, "Failed to update user password : " + sqlex.getMessage());
		}

		return result;
	}

	/***
	 * Check to see if it is acceptable to remove this user
	 * @param user
	 * @return
	 */
	public boolean canDeleteUser(User user) {
		boolean result = true;
		
		PreparedStatement stmnt = fDBConn.getStatementFor("canDeleteUser");
		
		try{
			stmnt.setInt(1, user.getUserID());
			
			stmnt.execute();
			ResultSet rs = stmnt.getResultSet();
			
			if (rs.next())
			{
				if (rs.getInt(1) > 0)
					result = false;
			}
				
			
			rs.close();
		} catch (SQLException sqlex)
		{
			Utility.log(Level.SEVERE, "Failed to retrieve users customization options : " + sqlex.getMessage());
		}
		
		return result;
	}
	
	public CustomizationOptions loadCustomizationOptions(User usr)
	{
		CustomizationOptions result = null;
		
		PreparedStatement stmnt = fDBConn.getStatementFor("loadCustomizationOptions");
		
		try {
			stmnt.setInt(1, usr.getUserID());
			
			if (stmnt.execute())
			{
				ResultSet rs = stmnt.getResultSet();
				result = new CustomizationOptions();
			
				if (rs.next())
				{
					result.setNickName(rs.getString("Nickname"));
					result.setGender(rs.getInt("Gender"));
					result.setHairStyle(rs.getInt("Hairstyle"));
					result.setSkinColor(rs.getInt("SkinColor"));
					result.setOutfitColor(rs.getInt("OutfitColor"));
				}
				
				rs.close();
			}
		} catch (SQLException sqlex)
		{
			Utility.log(Level.SEVERE, "Failed to retrieve users customization options : " + sqlex.getMessage());
		}
		
		return result;
	}
	
	public boolean saveCustomizationOptions(User usr, CustomizationOptions Options)
	{
		boolean result = false;
		PreparedStatement stmnt = fDBConn.getStatementFor("saveCustomizationOptions");
		
		try
		{
			stmnt.setString(1, Options.getNickname());
			stmnt.setInt(2, Options.getGender());
			stmnt.setInt(3, Options.getHairStyle());
			stmnt.setInt(4, Options.getSkinColor());
			stmnt.setInt(5, Options.getOutfitColor());
			stmnt.setInt(6, usr.getUserID());
			
			stmnt.execute();
			result = true;
		}
		catch (SQLException sqlex)
		{
			Utility.log(Level.SEVERE, "Failed to save users customization options : " + sqlex.getMessage());
		}
		return result;
	}
	
	public void submitWordScore(ExerciseWordScore score)
	{
		PreparedStatement stmnt = fDBConn.getStatementFor("submitWordScore");
		try
		{
			stmnt.setInt(1, score.getUserID());
			stmnt.setInt(2, score.getWordScore());
			stmnt.setInt(3, score.getExerciseWordID());
			
			stmnt.execute();
    	} catch (SQLException sqlex)
    	{
    		Utility.log(Level.SEVERE, "Failed to retrive user word scores : " + sqlex.getMessage());
    	}
	}
	
	public ArrayList<ExerciseWordScore> getUserWordScores(User usr)
	{
		ArrayList<ExerciseWordScore> result = new ArrayList<ExerciseWordScore>();
		
		PreparedStatement stmnt = fDBConn.getStatementFor("getUserWordScores");
		
		try
		{
			stmnt.setInt(1, usr.getUserID());
			
			stmnt.execute();
			
			ResultSet rs = stmnt.getResultSet();
			
			while (rs.next())
			{
				ExerciseWordScore tmp = new ExerciseWordScore();
				
				tmp.setExerciseWordID(rs.getInt("WordScoreID"));
				tmp.setWordScore(rs.getInt("Score"));
				tmp.setUserID(usr.getUserID());
				tmp.setWordText(rs.getString("Text"));
				tmp.setExerciseDescription(rs.getString("Name"));
				result.add(tmp);
			}
			
			rs.close();
		} catch (SQLException sqlex)
		{
			Utility.log(Level.SEVERE, "Failed to retrive user word scores : " + sqlex.getMessage());
		}
		
		return result;
	}
}
